@@ -1,31 +1,6 @@
 /* cache.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -34,6 +9,7 @@
 #include <string.h>
 #include <strings.h>
 #include <stdarg.h>
+#include <stdint.h>
 
 #include "cache.h"
 
@@ -52,7 +28,7 @@ static char* form_key(const char *s) {
     size_t	len = strlen(s);
     char	*d = ALLOC_N(char, len + 2);
 
-    *d = (255 <= len) ? 255 : len;
+    *(uint8_t*)d = (255 <= len) ? 255 : len;
     memcpy(d + 1, s, len + 1);
 
     return d;
@@ -67,7 +43,7 @@ ox_cache_new(Cache *cache) {
 }
 
 VALUE
-ox_cache_get(Cache cache, const char *key, VALUE **slot, char **keyp) {
+ox_cache_get(Cache cache, const char *key, VALUE **slot, const char **keyp) {
     unsigned char       *k = (unsigned char*)key;
     Cache               *cp;
 
@@ -131,7 +107,7 @@ ox_cache_get(Cache cache, const char *key, VALUE **slot, char **keyp) {
     *slot = &cache->value;
     if (0 != keyp) {
 	if (0 == cache->key) {
-	    printf("*** Error: failed to set the key for %s\n", key);
+	    printf("*** Error: failed to set the key for '%s'\n", key);
 	    *keyp = 0;
 	} else {
 	    *keyp = cache->key + 1;
