@@ -1,31 +1,6 @@
 /* dump.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -91,7 +66,7 @@ static void	dump_end(Out out, Element e);
 static void	grow(Out out, size_t len);
 
 static void	dump_value(Out out, const char *value, size_t size);
-static void	dump_str_value(Out out, const char *value, size_t size);
+static void	dump_str_value(Out out, const char *value, size_t size, const char *table);
 static int	dump_var(ID key, VALUE value, Out out);
 static void	dump_num(Out out, VALUE obj);
 static void	dump_date(Out out, VALUE obj);
@@ -99,12 +74,14 @@ static void	dump_time_thin(Out out, VALUE obj);
 static void	dump_time_xsd(Out out, VALUE obj);
 static int	dump_hash(VALUE key, VALUE value, Out out);
 
-static int	is_xml_friendly(const uchar *str, int len);
+static int	is_xml_friendly(const uchar *str, int len, const char *table);
 
 static const char	hex_chars[17] = "0123456789abcdef";
 
-static char	xml_friendly_chars[257] = "\
-88888888811881888888888888888888\
+// The : character is equivalent to 10. Used for replacement characters up to 10
+// characters long such as '&#x10FFFF;'.
+static const char	xml_friendly_chars[257] = "\
+:::::::::11::1::::::::::::::::::\
 11611156111111111111111111114141\
 11111111111111111111111111111111\
 11111111111111111111111111111111\
@@ -113,10 +90,30 @@ static char	xml_friendly_chars[257] = "\
 11111111111111111111111111111111\
 11111111111111111111111111111111";
 
+static const char	xml_quote_chars[257] = "\
+:::::::::11::1::::::::::::::::::\
+11611151111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111";
+
+static const char	xml_element_chars[257] = "\
+:::::::::11::1::::::::::::::::::\
+11111151111111111111111111114141\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111";
+
 inline static int
-is_xml_friendly(const uchar *str, int len) {
+is_xml_friendly(const uchar *str, int len, const char *table) {
     for (; 0 < len; str++, len--) {
-	if ('1' != xml_friendly_chars[*str]) {
+	if ('1' != table[*str]) {
 	    return 0;
 	}
     }
@@ -124,7 +121,7 @@ is_xml_friendly(const uchar *str, int len) {
 }
 
 inline static size_t
-xml_str_len(const uchar *str, size_t len) {
+xml_str_len(const uchar *str, size_t len, const char *table) {
     size_t	size = 0;
 
     for (; 0 < len; str++, len--) {
@@ -154,12 +151,12 @@ obj_class_code(VALUE obj) {
     case T_FALSE:	   return FalseClassCode;
     case T_FIXNUM:	   return FixnumCode;
     case T_FLOAT:	   return FloatCode;
-    case T_STRING:	   return (is_xml_friendly((uchar*)StringValuePtr(obj), (int)RSTRING_LEN(obj))) ? StringCode : String64Code;
+    case T_STRING:	   return (is_xml_friendly((uchar*)StringValuePtr(obj), (int)RSTRING_LEN(obj), xml_element_chars)) ? StringCode : String64Code;
     case T_SYMBOL:
     {
 	const char	*sym = rb_id2name(SYM2ID(obj));
 
-	return (is_xml_friendly((uchar*)sym, (int)strlen(sym))) ? SymbolCode : Symbol64Code;
+	return (is_xml_friendly((uchar*)sym, (int)strlen(sym), xml_element_chars)) ? SymbolCode : Symbol64Code;
     }
     case T_DATA:	   return (rb_cTime == clas) ? TimeCode : ((ox_date_class == clas) ? DateCode : 0);
     case T_STRUCT:	   return (rb_cRange == clas) ? RangeCode : StructCode;
@@ -181,6 +178,10 @@ inline static void
 fill_indent(Out out, int cnt) {
     if (0 <= cnt) {
 	*out->cur++ = '\n';
+	if (0 < out->opts->margin_len) {
+	    memcpy(out->cur, out->opts->margin, out->opts->margin_len);
+	    out->cur += out->opts->margin_len;
+	}
 	for (; 0 < cnt; cnt--) {
 	    *out->cur++ = ' ';
 	}
@@ -267,7 +268,7 @@ grow(Out out, size_t len) {
 
 static void
 dump_start(Out out, Element e) {
-    size_t	size = e->indent + 4;
+    size_t	size = e->indent + 4 + out->opts->margin_len;
 
     if (0 < e->attr.len) { /* a="attr" */
 	size += e->attr.len + 5;
@@ -281,7 +282,7 @@ dump_start(Out out, Element e) {
     if (out->end - out->cur <= (long)size) {
 	grow(out, size);
     }
-    if (out->buf < out->cur) {
+    if (out->buf + out->opts->margin_len < out->cur) {
 	fill_indent(out, e->indent);
     }
     *out->cur++ = '<';
@@ -308,7 +309,7 @@ dump_start(Out out, Element e) {
 
 static void
 dump_end(Out out, Element e) {
-    size_t	size = e->indent + 5;
+    size_t	size = e->indent + 5 + out->opts->margin_len;
 
     if (out->end - out->cur <= (long)size) {
 	grow(out, size);
@@ -338,52 +339,73 @@ dump_value(Out out, const char *value, size_t size) {
 }
 
 inline static void
-dump_str_value(Out out, const char *value, size_t size) {
-    size_t	xsize = xml_str_len((const uchar*)value, size);
-
+dump_str_value(Out out, const char *value, size_t size, const char *table) {
+    size_t	xsize = xml_str_len((const uchar*)value, size, table);
+    
     if (out->end - out->cur <= (long)xsize) {
 	grow(out, xsize);
     }
     for (; '\0' != *value; value++) {
-	if ('1' == xml_friendly_chars[(uchar)*value]) {
+	if ('1' == table[(uchar)*value]) {
 	    *out->cur++ = *value;
 	} else {
-	    *out->cur++ = '&';
 	    switch (*value) {
 	    case '"':
+		*out->cur++ = '&';
 		*out->cur++ = 'q';
 		*out->cur++ = 'u';
 		*out->cur++ = 'o';
 		*out->cur++ = 't';
+		*out->cur++ = ';';
 		break;
 	    case '&':
+		*out->cur++ = '&';
 		*out->cur++ = 'a';
 		*out->cur++ = 'm';
 		*out->cur++ = 'p';
+		*out->cur++ = ';';
 		break;
 	    case '\'':
+		*out->cur++ = '&';
 		*out->cur++ = 'a';
 		*out->cur++ = 'p';
 		*out->cur++ = 'o';
 		*out->cur++ = 's';
+		*out->cur++ = ';';
 		break;
 	    case '<':
+		*out->cur++ = '&';
 		*out->cur++ = 'l';
 		*out->cur++ = 't';
+		*out->cur++ = ';';
 		break;
 	    case '>':
+		*out->cur++ = '&';
 		*out->cur++ = 'g';
 		*out->cur++ = 't';
+		*out->cur++ = ';';
 		break;
 	    default:
-		*out->cur++ = '#';
-		*out->cur++ = 'x';
-		*out->cur++ = '0';
-		*out->cur++ = '0';
-		dump_hex(*value, out);
+		// Must be one of the invalid characters.
+		if (StrictEffort == out->opts->effort) {
+		    rb_raise(rb_eSyntaxError, "'\\#x%02x' is not a valid XML character.", *value);
+		}
+		if (Yes == out->opts->allow_invalid) {
+		    *out->cur++ = '&';
+		    *out->cur++ = '#';
+		    *out->cur++ = 'x';
+		    *out->cur++ = '0';
+		    *out->cur++ = '0';
+		    dump_hex(*value, out);
+		    *out->cur++ = ';';
+		} else if ('\0' != *out->opts->inv_repl) {
+		    // If the empty string then ignore. The first character of
+		    // the replacement is the length.
+		    memcpy(out->cur, out->opts->inv_repl + 1, (size_t)*out->opts->inv_repl);
+		    out->cur += *out->opts->inv_repl;
+		}
 		break;
 	    }
-	    *out->cur++ = ';';
 	}
     }
     *out->cur = '\0';
@@ -532,24 +554,38 @@ dump_first_obj(VALUE obj, Out out) {
     int		cnt;
 
     if (Yes == copts->with_xml) {
+	if (0 < copts->margin_len) {
+	    dump_value(out, copts->margin, copts->margin_len);
+	}
 	if ('\0' == *copts->encoding) {
 	    dump_value(out, "<?xml version=\"1.0\"?>", 21);
 	} else {
-	    cnt = sprintf(buf, "<?xml version=\"1.0\" encoding=\"%s\"?>", copts->encoding);
+	    cnt = snprintf(buf, sizeof(buf), "<?xml version=\"1.0\" encoding=\"%s\"?>", copts->encoding);
 	    dump_value(out, buf, cnt);
 	}
     }
     if (Yes == copts->with_instruct) {
-	cnt = sprintf(buf, "%s<?ox version=\"1.0\" mode=\"object\"%s%s?>",
-		      (out->buf < out->cur) ? "\n" : "",
+	if (out->buf < out->cur) {
+	    dump_value(out, "\n", 1);
+	}
+	if (0 < copts->margin_len) {
+	    dump_value(out, copts->margin, copts->margin_len);
+	}
+	cnt = snprintf(buf, sizeof(buf), "<?ox version=\"1.0\" mode=\"object\"%s%s?>",
 		      (Yes == copts->circular) ? " circular=\"yes\"" : ((No == copts->circular) ? " circular=\"no\"" : ""),
 		      (Yes == copts->xsd_date) ? " xsd_date=\"yes\"" : ((No == copts->xsd_date) ? " xsd_date=\"no\"" : ""));
 	dump_value(out, buf, cnt);
     }
     if (Yes == copts->with_dtd) {
-	cnt = sprintf(buf, "%s<!DOCTYPE %c SYSTEM \"ox.dtd\">", (out->buf < out->cur) ? "\n" : "", obj_class_code(obj));
+	if (0 < copts->margin_len) {
+	    dump_value(out, copts->margin, copts->margin_len);
+	}
+	cnt = snprintf(buf, sizeof(buf), "%s<!DOCTYPE %c SYSTEM \"ox.dtd\">", (out->buf < out->cur) ? "\n" : "", obj_class_code(obj));
 	dump_value(out, buf, cnt);
     }
+    if (0 < copts->margin_len) {
+	dump_value(out, copts->margin, copts->margin_len);
+    }
     dump_obj(0, obj, 0, out);
 }
 
@@ -565,10 +601,15 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
     }
     out->obj = obj;
     if (0 == aid) {
-	/*e.attr.str = 0; */
+	e.attr.str = 0;
 	e.attr.len = 0;
     } else {
 	e.attr.str = rb_id2name(aid);
+	// Ruby 2.3 started to return NULL for some IDs so check for
+	// NULL. Ignore if NULL aid.
+	if (NULL == e.attr.str) {
+	    return;
+	}
 	e.attr.len = strlen(e.attr.str);
     }
     e.closed = 0;
@@ -645,7 +686,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	break;
     case T_FLOAT:
 	e.type = FloatCode;
-	cnt = sprintf(value_buf, "%0.16g", rb_num2dbl(obj)); /* used sprintf due to bug in snprintf */
+	cnt = snprintf(value_buf, sizeof(value_buf), "%0.16g", rb_num2dbl(obj));
 	out->w_start(out, &e);
 	dump_value(out, value_buf, cnt);
 	e.indent = -1;
@@ -664,7 +705,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	if (is_xml_friendly((uchar*)str, cnt)) {
 	    e.type = StringCode;
 	    out->w_start(out, &e);
-	    dump_str_value(out, str, cnt);
+	    dump_str_value(out, str, cnt, '<');
 	    e.indent = -1;
 	    out->w_end(out, &e);
 	} else {
@@ -681,7 +722,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 #else
 	e.type = StringCode;
 	out->w_start(out, &e);
-	dump_str_value(out, str, cnt);
+	dump_str_value(out, str, cnt, xml_element_chars);
 	e.indent = -1;
 	out->w_end(out, &e);
 #endif
@@ -696,7 +737,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	if (is_xml_friendly((uchar*)sym, cnt)) {
 	    e.type = SymbolCode;
 	    out->w_start(out, &e);
-	    dump_str_value(out, sym, cnt);
+	    dump_str_value(out, sym, cnt, '<');
 	    e.indent = -1;
 	    out->w_end(out, &e);
 	} else {
@@ -713,7 +754,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 #else
 	e.type = SymbolCode;
 	out->w_start(out, &e);
-	dump_str_value(out, sym, cnt);
+	dump_str_value(out, sym, cnt, xml_element_chars);
 	e.indent = -1;
 	out->w_end(out, &e);
 #endif
@@ -769,9 +810,9 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	}
 	clas = rb_obj_class(obj);
 	if (rb_cRange == clas) {
-	    VALUE	beg = RSTRUCT_PTR(obj)[0];
-	    VALUE	end = RSTRUCT_PTR(obj)[1];
-	    VALUE	excl = RSTRUCT_PTR(obj)[2];
+	    VALUE	beg  = RSTRUCT_GET(obj, 0);
+	    VALUE	end  = RSTRUCT_GET(obj, 1);
+	    VALUE	excl = RSTRUCT_GET(obj, 2);
 	    int		d2 = depth + 1;
 
 	    e.type = RangeCode;	 e.clas.len = 5;  e.clas.str = "Range";
@@ -782,17 +823,22 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	    out->w_end(out, &e);
 	} else {
 	    char	num_buf[16];
-	    VALUE	*vp;
-	    int		i;
 	    int		d2 = depth + 1;
-	    
+#if UNIFY_FIXNUM_AND_BIGNUM
+		long i;
+		long cnt = NUM2LONG(rb_struct_size(obj));
+#else // UNIFY_FIXNUM_AND_INTEGER
+		int   i;
+		int   cnt = (int)RSTRUCT_LEN(obj);
+#endif // UNIFY_FIXNUM_AND_INTEGER
 	    e.type = StructCode;
 	    e.clas.str = rb_class2name(clas);
 	    e.clas.len = strlen(e.clas.str);
 	    out->w_start(out, &e);
-	    cnt = (int)RSTRUCT_LEN(obj);
-	    for (i = 0, vp = RSTRUCT_PTR(obj); i < cnt; i++, vp++) {
-		dump_obj(rb_intern(ulong2str(i, num_buf + sizeof(num_buf) - 1)), *vp, d2, out);
+
+	    for (i = 0; i < cnt; i++) {
+	        VALUE v = RSTRUCT_GET(obj, i);
+	        dump_obj(rb_intern(ulong2str(i, num_buf + sizeof(num_buf) - 1)), v, d2, out);
 	    }
 	    out->w_end(out, &e);
 	}
@@ -874,7 +920,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 #if USE_B64
 	if (is_xml_friendly((uchar*)s, cnt)) {
 	    /*dump_value(out, "/", 1); */
-	    dump_str_value(out, s, cnt);
+	    dump_str_value(out, s, cnt, '<');
 	} else {
 	    ulong	size = b64_size(cnt);
 	    char	*b64 = ALLOCA_N(char, size + 1);
@@ -883,7 +929,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	    dump_value(out, b64, size);
 	}
 #else
-	dump_str_value(out, s, cnt);
+	dump_str_value(out, s, cnt, xml_element_chars);
 #endif
 	e.indent = -1;
 	out->w_end(out, &e);
@@ -991,6 +1037,9 @@ dump_gen_doc(VALUE obj, int depth, Out out) {
 	}
     }
     if (Yes == out->opts->with_xml) {
+	if (0 < out->opts->margin_len) {
+	    dump_value(out, out->opts->margin, out->opts->margin_len);
+	}
 	dump_value(out, "<?xml", 5);
 	if (Qnil != attrs) {
 	    rb_hash_foreach(attrs, dump_gen_attr, (VALUE)out);
@@ -999,10 +1048,12 @@ dump_gen_doc(VALUE obj, int depth, Out out) {
     }
     if (Yes == out->opts->with_instruct) {
 	if (out->buf < out->cur) {
-	    dump_value(out, "\n<?ox version=\"1.0\" mode=\"generic\"?>", 36);
-	} else {
-	    dump_value(out, "<?ox version=\"1.0\" mode=\"generic\"?>", 35);
+	    dump_value(out, "\n", 1);
 	}
+	if (0 < out->opts->margin_len) {
+	    dump_value(out, out->opts->margin, out->opts->margin_len);
+	}
+	dump_value(out, "<?ox version=\"1.0\" mode=\"generic\"?>", 35);
     }
     if (Qnil != nodes) {
 	dump_gen_nodes(nodes, depth, out);
@@ -1026,17 +1077,21 @@ dump_gen_element(VALUE obj, int depth, Out out) {
     } else {
 	indent = depth * out->indent;
     }
-    size = indent + 4 + nlen;
+    size = indent + 4 + nlen + out->opts->margin_len;
     if (out->end - out->cur <= (long)size) {
 	grow(out, size);
     }
+    if (0 == depth && 0 < out->opts->margin_len && 0 < out->indent) {
+	memcpy(out->cur, out->opts->margin, out->opts->margin_len);
+	out->cur += out->opts->margin_len;
+    }
     fill_indent(out, indent);
     *out->cur++ = '<';
     fill_value(out, name, nlen);
     if (Qnil != attrs) {
 	rb_hash_foreach(attrs, dump_gen_attr, (VALUE)out);
     }
-    if (Qnil != nodes) {
+    if (Qnil != nodes && 0 < RARRAY_LEN(nodes)) {
 	int	do_indent;
 	
 	*out->cur++ = '>';
@@ -1112,10 +1167,12 @@ dump_gen_nodes(VALUE obj, int depth, Out out) {
 		dump_gen_instruct(*np, d2, out);
 		indent_needed = (1 == cnt) ? 0 : 1;
 	    } else if (rb_cString == clas) {
-		dump_str_value(out, StringValuePtr(*(VALUE*)np), RSTRING_LEN(*np));
+		dump_str_value(out, StringValuePtr(*(VALUE*)np), RSTRING_LEN(*np), xml_element_chars);
 		indent_needed = (1 == cnt) ? 0 : 1;
 	    } else if (ox_comment_clas == clas) {
 		dump_gen_val_node(*np, d2, "<!-- ", 5, " -->", 4, out);
+	    } else if (ox_raw_clas == clas) {
+		dump_gen_val_node(*np, d2, "", 0, "", 0, out);
 	    } else if (ox_cdata_clas == clas) {
 		dump_gen_val_node(*np, d2, "<![CDATA[", 9, "]]>", 3, out);
 	    } else if (ox_doctype_clas == clas) {
@@ -1162,7 +1219,7 @@ dump_gen_attr(VALUE key, VALUE value, Out out) {
     fill_value(out, ks, klen);
     *out->cur++ = '=';
     *out->cur++ = '"';
-    dump_str_value(out, StringValuePtr(value), RSTRING_LEN(value));
+    dump_str_value(out, StringValuePtr(value), RSTRING_LEN(value), xml_quote_chars);
     *out->cur++ = '"';
 
     return ST_CONTINUE;
@@ -1190,7 +1247,7 @@ dump_gen_val_node(VALUE obj, int depth,
     } else {
 	indent = depth * out->indent;
     }
-    size = indent + plen + slen + vlen;
+    size = indent + plen + slen + vlen + out->opts->margin_len;
     if (out->end - out->cur <= (long)size) {
 	grow(out, size);
     }
@@ -1217,6 +1274,7 @@ dump_obj_to_xml(VALUE obj, Options copts, Out out) {
 	ox_cache8_new(&out->circ_cache);
     }
     out->indent = copts->indent;
+
     if (ox_document_clas == clas) {
 	dump_gen_doc(obj, -1, out);
     } else if (ox_element_clas == clas) {
