@@ -1,31 +1,6 @@
 /* gen_load.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -59,6 +34,7 @@ struct _ParseCallbacks   _ox_gen_callbacks = {
     add_text,
     add_element,
     end_element,
+    NULL,
 };
 
 ParseCallbacks   ox_gen_callbacks = &_ox_gen_callbacks;
@@ -71,6 +47,7 @@ struct _ParseCallbacks   _ox_limited_callbacks = {
     add_text,
     add_element,
     end_element,
+    NULL,
 };
 
 ParseCallbacks   ox_limited_callbacks = &_ox_limited_callbacks;
@@ -83,6 +60,7 @@ struct _ParseCallbacks   _ox_nomode_callbacks = {
     add_text,
     add_element,
     end_element,
+    NULL,
 };
 
 ParseCallbacks   ox_nomode_callbacks = &_ox_nomode_callbacks;
@@ -325,13 +303,14 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
     e = rb_obj_alloc(ox_element_clas);
     rb_ivar_set(e, ox_at_value_id, s);
     if (0 != attrs->name) {
-        VALUE   ah = rb_hash_new();
+        volatile VALUE	ah = rb_hash_new();
         
         for (; 0 != attrs->name; attrs++) {
-            VALUE   sym;
-            VALUE   *slot;
+            volatile VALUE	sym;
 
 	    if (Yes == pi->options->sym_keys) {
+		VALUE	*slot;
+
 		if (Qundef == (sym = ox_cache_get(ox_symbol_cache, attrs->name, &slot, 0))) {
 #if HAS_ENCODING_SUPPORT
 		    if (0 != pi->options->rb_enc) {
@@ -354,6 +333,9 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
 #else
 		    sym = ID2SYM(rb_intern(attrs->name));
 #endif
+		    // Needed for Ruby 2.2 to get around the GC of symbols
+		    // created with to_sym which is needed for encoded symbols.
+		    rb_ary_push(ox_sym_bank, sym);
 		    *slot = sym;
 		}
 	    } else {
@@ -462,6 +444,9 @@ add_instruct(PInfo pi, const char *name, Attr attrs, const char *content) {
 #else
 		    sym = ID2SYM(rb_intern(attrs->name));
 #endif
+		    // Needed for Ruby 2.2 to get around the GC of symbols
+		    // created with to_sym which is needed for encoded symbols.
+		    rb_ary_push(ox_sym_bank, sym);
 		    *slot = sym;
 		}
 	    } else {
