@@ -1,31 +1,6 @@
 /* obj_load.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -71,6 +46,7 @@ struct _ParseCallbacks	 _ox_obj_callbacks = {
     add_text,
     add_element,
     end_element,
+    NULL,
 };
 
 ParseCallbacks	 ox_obj_callbacks = &_ox_obj_callbacks;
@@ -107,11 +83,14 @@ name2var(const char *name, void *encoding) {
     } else if (Qundef == (var_id = ox_cache_get(ox_attr_cache, name, &slot, 0))) {
 #ifdef HAVE_RUBY_ENCODING_H
 	if (0 != encoding) {
-	    VALUE	rstr = rb_str_new2(name);
-	    VALUE	sym;
+	    volatile VALUE	rstr = rb_str_new2(name);
+	    volatile VALUE	sym;
 	    
 	    rb_enc_associate(rstr, (rb_encoding*)encoding);
 	    sym = rb_funcall(rstr, ox_to_sym_id, 0);
+	    // Needed for Ruby 2.2 to get around the GC of symbols
+	    // created with to_sym which is needed for encoded symbols.
+	    rb_ary_push(ox_sym_bank, sym);
 	    var_id = SYM2ID(sym);
 	} else {
 	    var_id = rb_intern(name);
@@ -487,6 +466,9 @@ add_text(PInfo pi, char *text, int closed) {
 
 	if (Qundef == (sym = ox_cache_get(ox_symbol_cache, text, &slot, 0))) {
 	    sym = str2sym(text, (void*)pi->options->rb_enc);
+	    // Needed for Ruby 2.2 to get around the GC of symbols created with
+	    // to_sym which is needed for encoded symbols.
+	    rb_ary_push(ox_sym_bank, sym);
 	    *slot = sym;
 	}
 	h->obj = sym;
@@ -538,6 +520,9 @@ add_text(PInfo pi, char *text, int closed) {
 	from_base64(text, (uchar*)str);
 	if (Qundef == (sym = ox_cache_get(ox_symbol_cache, str, &slot, 0))) {
 	    sym = str2sym(str, (void*)pi->options->rb_enc);
+	    // Needed for Ruby 2.2 to get around the GC of symbols created with
+	    // to_sym which is needed for encoded symbols.
+	    rb_ary_push(ox_sym_bank, sym);
 	    *slot = sym;
 	}
 	h->obj = sym;
@@ -772,11 +757,11 @@ end_element(PInfo pi, const char *ename) {
 	    case RangeCode:
 #if HAS_RSTRUCT
 		if (ox_beg_id == h->var) {
-		    RSTRUCT_PTR(ph->obj)[0] = h->obj;
+		    RSTRUCT_SET(ph->obj, 0, h->obj);
 		} else if (ox_end_id == h->var) {
-		    RSTRUCT_PTR(ph->obj)[1] = h->obj;
+		    RSTRUCT_SET(ph->obj, 1, h->obj);
 		} else if (ox_excl_id == h->var) {
-		    RSTRUCT_PTR(ph->obj)[2] = h->obj;
+		    RSTRUCT_SET(ph->obj, 2, h->obj);
 		} else {
 		    set_error(&pi->err, "Invalid range attribute", pi->str, pi->s);
 		    return;
@@ -791,8 +776,10 @@ end_element(PInfo pi, const char *ename) {
 		    Helper	gh;
 
 		    helper_stack_pop(&pi->helpers);
-		    gh = helper_stack_peek(&pi->helpers);
-
+		    if (NULL == (gh = helper_stack_peek(&pi->helpers))) {
+			set_error(&pi->err, "Corrupt parse stack, container is wrong type", pi->str, pi->s);
+			return;
+		    }
 		    rb_hash_aset(gh->obj, ph->obj, h->obj);
 		}
 		break;
