@@ -1,31 +1,6 @@
 /* ox.h
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- * - Redistributions of source code must retain the above copyright notice, this
- *   list of conditions and the following disclaimer.
- * 
- * - Redistributions in binary form must reproduce the above copyright notice,
- *   this list of conditions and the following disclaimer in the documentation
- *   and/or other materials provided with the distribution.
- * 
- * - Neither the name of Peter Ohler nor the names of its contributors may be
- *   used to endorse or promote products derived from this software without
- *   specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef __OX_H__
@@ -112,12 +87,21 @@ typedef enum {
 } YesNo;
 
 typedef enum {
-    ObjMode  = 'o',
-    GenMode  = 'g',
-    LimMode  = 'l',
-    NoMode   = 0
+    ObjMode        = 'o',
+    GenMode        = 'g',
+    LimMode        = 'l',
+    HashMode       = 'h',
+    HashNoAttrMode = 'n',
+    NoMode         = 0
 } LoadMode;
 
+typedef enum {
+    OffSkip  = 'o',
+    NoSkip   = 'n',
+    CrSkip   = 'r',
+    SpcSkip  = 's',
+} SkipMode;
+
 typedef struct _PInfo	*PInfo;
 
 typedef struct _ParseCallbacks {
@@ -128,6 +112,7 @@ typedef struct _ParseCallbacks {
     void	(*add_text)(PInfo pi, char *text, int closed);
     void	(*add_element)(PInfo pi, const char *ename, Attr attrs, int hasChildren);
     void	(*end_element)(PInfo pi, const char *ename);
+    void	(*finish)(PInfo pi);
 } *ParseCallbacks;
 
 typedef struct _CircArray {
@@ -138,23 +123,32 @@ typedef struct _CircArray {
 } *CircArray;
 
 typedef struct _Options {
-    char	encoding[64];	/* encoding, stored in the option to avoid GC invalidation in default values */
-    int		indent;		/* indention for dump, default 2 */
-    int		trace;		/* trace level */
-    char	with_dtd;	/* YesNo */
-    char	with_xml;	/* YesNo */
-    char	with_instruct;	/* YesNo */
-    char	circular;	/* YesNo */
-    char	xsd_date;	/* YesNo */
-    char	mode;		/* LoadMode */
-    char	effort;		/* Effort */
-    char	sym_keys;	/* symbolize keys */
+    char		encoding[64];	/* encoding, stored in the option to avoid GC invalidation in default values */
+    char		margin[128];	/* left margin for dumping */
+    int			indent;		/* indention for dump, default 2 */
+    int			trace;		/* trace level */
+    char		margin_len;	/* margin length */
+    char		with_dtd;	/* YesNo */
+    char		with_xml;	/* YesNo */
+    char		with_instruct;	/* YesNo */
+    char		circular;	/* YesNo */
+    char		xsd_date;	/* YesNo */
+    char		mode;		/* LoadMode */
+    char		effort;		/* Effort */
+    char		sym_keys;	/* symbolize keys */
+    char		skip;		/* skip mode */
+    char		smart;		/* YesNo sax smart mode */
+    char		convert_special;/* boolean true or false */
+    char		allow_invalid;	/* YesNo */
+    char		inv_repl[12];	/* max 10 valid characters, first character is the length */
+    char		strip_ns[64];	/* namespace to strip, \0 is no-strip, \* is all, else only matches */
+    struct _Hints	*html_hints;	/* html hints */
 #if HAS_ENCODING_SUPPORT
-    rb_encoding	*rb_enc;
+    rb_encoding		*rb_enc;
 #elif HAS_PRIVATE_ENCODING
-    VALUE	rb_enc;
+    VALUE		rb_enc;
 #else
-    void	*rb_enc;
+    void		*rb_enc;
 #endif
 } *Options;
 
@@ -184,10 +178,12 @@ extern struct _Options	ox_default_options;
 
 extern VALUE	Ox;
 
+extern ID	ox_abort_id;
 extern ID	ox_at_column_id;
 extern ID	ox_at_content_id;
 extern ID	ox_at_id;
 extern ID	ox_at_line_id;
+extern ID	ox_at_pos_id;
 extern ID	ox_at_value_id;
 extern ID	ox_attr_id;
 extern ID	ox_attr_value_id;
@@ -213,10 +209,11 @@ extern ID	ox_keys_id;
 extern ID	ox_local_id;
 extern ID	ox_mesg_id;
 extern ID	ox_message_id;
-extern ID	ox_nodes_id;
 extern ID	ox_new_id;
+extern ID	ox_nodes_id;
 extern ID	ox_num_id;
 extern ID	ox_parse_id;
+extern ID	ox_pos_id;
 extern ID	ox_read_id;
 extern ID	ox_readpartial_id;
 extern ID	ox_start_element_id;
@@ -238,20 +235,27 @@ extern VALUE		ox_utf8_encoding;
 extern void		*ox_utf8_encoding;
 #endif
 
-extern VALUE	ox_bigdecimal_class;
-extern VALUE	ox_date_class;
 extern VALUE	ox_empty_string;
 extern VALUE	ox_encoding_sym;
+extern VALUE	ox_indent_sym;
+extern VALUE	ox_size_sym;
+extern VALUE	ox_standalone_sym;
+extern VALUE	ox_sym_bank; // Array
+extern VALUE	ox_version_sym;
+extern VALUE	ox_zero_fixnum;
+
+extern VALUE	ox_bigdecimal_class;
+extern VALUE	ox_date_class;
 extern VALUE	ox_stringio_class;
 extern VALUE	ox_struct_class;
 extern VALUE	ox_time_class;
-extern VALUE	ox_zero_fixnum;
 
 extern VALUE	ox_document_clas;
 extern VALUE	ox_element_clas;
 extern VALUE	ox_instruct_clas;
 extern VALUE	ox_bag_clas;
 extern VALUE	ox_comment_clas;
+extern VALUE	ox_raw_clas;
 extern VALUE	ox_doctype_clas;
 extern VALUE	ox_cdata_clas;
 
@@ -259,6 +263,8 @@ extern Cache	ox_symbol_cache;
 extern Cache	ox_class_cache;
 extern Cache	ox_attr_cache;
 
+extern void	ox_init_builder(VALUE ox);
+
 #if defined(__cplusplus)
 #if 0
 { /* satisfy cc-mode */
