@@ -1,31 +1,6 @@
 /* parse.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -68,6 +43,23 @@ static int	collapse_special(PInfo pi, char *str);
  * all cases to parse the string.
  */
 
+static char	xml_valid_lower_chars[34] = "xxxxxxxxxooxxoxxxxxxxxxxxxxxxxxxo";
+
+inline static int
+is_white(char c) {
+    switch (c) {
+    case ' ':
+    case '\t':
+    case '\f':
+    case '\n':
+    case '\r':
+	return 1;
+    default:
+	return 0;
+    }
+}
+
+
 inline static void
 next_non_white(PInfo pi) {
     for (; 1; pi->s++) {
@@ -101,11 +93,27 @@ next_white(PInfo pi) {
     }
 }
 
+static void
+mark_pi_cb(void *ptr) {
+    if (NULL != ptr) {
+	HelperStack	stack = &((PInfo)ptr)->helpers;
+	Helper		h;
+
+	for (h = stack->head; h < stack->tail; h++) {
+	    if (NoCode != h->type) {
+		rb_gc_mark(h->obj);
+	    }
+	}
+    }
+}
+
 VALUE
 ox_parse(char *xml, ParseCallbacks pcb, char **endp, Options options, Err err) {
     struct _PInfo	pi;
     int			body_read = 0;
-
+    int			block_given = rb_block_given_p();
+    volatile VALUE	wrap;
+    
     if (0 == xml) {
 	set_error(err, "Invalid arg, xml string can not be null", xml, 0);
 	return Qnil;
@@ -115,6 +123,9 @@ ox_parse(char *xml, ParseCallbacks pcb, char **endp, Options options, Err err) {
     }
     /* initialize parse info */
     helper_stack_init(&pi.helpers);
+    // Protect against GC
+    wrap = Data_Wrap_Struct(rb_cObject, mark_pi_cb, NULL, &pi);
+
     err_init(&pi.err);
     pi.str = xml;
     pi.s = xml;
@@ -181,8 +192,18 @@ ox_parse(char *xml, ParseCallbacks pcb, char **endp, Options options, Err err) {
 	    helper_stack_cleanup(&pi.helpers);
 	    return Qnil;
 	}
+	if (block_given && Qnil != pi.obj && Qundef != pi.obj) {
+	    if (NULL != pcb->finish) {
+		pcb->finish(&pi);
+	    }
+	    rb_yield(pi.obj);
+	}
     }
+    DATA_PTR(wrap) = NULL;
     helper_stack_cleanup(&pi.helpers);
+    if (NULL != pcb->finish) {
+	pcb->finish(&pi);
+    }
     return pi.obj;
 }
 
@@ -221,7 +242,6 @@ read_instruction(PInfo pi) {
     char		*cend;
     int			attrs_ok = 1;
 
-
     *content = '\0';
     attr_stack_init(&attrs);
     if (0 == (target = read_name_token(pi))) {
@@ -495,10 +515,9 @@ read_element(PInfo pi) {
 	    if (0 == (attr_value = read_quoted_value(pi))) {
 		return 0;
 	    }
-	    if (0 != strchr(attr_value, '&')) {
-		if (0 != collapse_special(pi, (char*)attr_value)) {
+	    if (pi->options->convert_special && 0 != strchr(attr_value, '&')) {
+		if (0 != collapse_special(pi, (char*)attr_value) || err_has(&pi->err)) {
 		    attr_stack_cleanup(&attrs);
-		    set_error(&pi->err, "invalid format, special character does not end with a semicolon", pi->str, pi->s);
 		    return 0;
 		}
 	    }
@@ -562,7 +581,7 @@ read_element(PInfo pi) {
 		    next_non_white(pi);
 		    c = *pi->s;
 		    *end = '\0';
-		    if (0 != strcmp(name, ename)) {
+		    if (0 != ((TolerantEffort == pi->options->effort) ? strcasecmp(name, ename) : strcmp(name, ename))) {
 			attr_stack_cleanup(&attrs);
 			if (TolerantEffort == pi->options->effort) {
 			    pi->pcb->end_element(pi, ename);
@@ -578,9 +597,33 @@ read_element(PInfo pi) {
 			return 0;
 		    }
 		    if (first && start != slash - 1) {
-			/* some white space between start and here so add as text */
+			// Some white space between start and here so add as
+			// text after checking skip.
 			*(slash - 1) = '\0';
-			pi->pcb->add_text(pi, start, 1);
+			switch (pi->options->skip) {
+			case CrSkip: {
+			    char	*s = start;
+			    char	*e = start;
+			
+			    for (; '\0' != *e; e++) {
+				if ('\r' != *e) {
+				    *s++ = *e;
+				}
+			    }
+			    *s = '\0';
+			    break;
+			}
+			case SpcSkip:
+			    *start = '\0';
+			    break;
+			case NoSkip:
+			case OffSkip:
+			default:
+			    break;
+			}
+			if ('\0' != *start) {
+			    pi->pcb->add_text(pi, start, 1);
+			}
 		    }
 		    pi->s++;
 		    pi->pcb->end_element(pi, ename);
@@ -600,7 +643,8 @@ read_element(PInfo pi) {
 		    // Child closed with mismatched name.
 		    if (0 != (name = read_element(pi))) {
 			attr_stack_cleanup(&attrs);
-			if (0 == strcmp(name, ename)) {
+
+			if (0 == ((TolerantEffort == pi->options->effort) ? strcasecmp(name, ename) : strcmp(name, ename))) {
 			    pi->s++;
 			    pi->pcb->end_element(pi, ename);
 			    return 0;
@@ -621,8 +665,9 @@ read_element(PInfo pi) {
 
 		/* to exit read_text with no errors the next character must be < */
 		if ('/' == *(pi->s + 1) &&
-		    0 == strncmp(ename, pi->s + 2, elen) &&
+		    0 == ((TolerantEffort == pi->options->effort) ? strncasecmp(ename, pi->s + 2, elen) : strncmp(ename, pi->s + 2, elen)) &&
 		    '>' == *(pi->s + elen + 2)) {
+
 		    /* close tag after text so treat as a value */
 		    pi->s += elen + 3;
 		    pi->pcb->end_element(pi, ename);
@@ -678,7 +723,37 @@ read_text(PInfo pi) {
 		    return;
 		}
 	    } else {
-		*b++ = c;
+		if (0 <= c && c <= 0x20) {
+		    if (StrictEffort == pi->options->effort && 'x' == xml_valid_lower_chars[(unsigned char)c]) {
+			set_error(&pi->err, "invalid character", pi->str, pi->s);
+			return;
+		    }
+		    switch (pi->options->skip) {
+		    case CrSkip:
+			if (buf != b && '\n' == c && '\r' == *(b - 1)) {
+			    *(b - 1) = '\n';
+			} else {
+			    *b++ = c;
+			}
+			break;
+		    case SpcSkip:
+			if (is_white(c)) {
+			    if (buf == b || ' ' != *(b - 1)) {
+				*b++ = ' ';
+			    }
+			} else {
+			    *b++ = c;
+			}			
+			break;
+		    case NoSkip:
+		    case OffSkip:
+		    default:
+			*b++ = c;
+			break;
+		    }
+		} else {
+		    *b++ = c;
+		}
 	    }
 	    break;
 	}
@@ -785,6 +860,15 @@ read_name_token(PInfo pi) {
 	    set_error(&pi->err, "invalid format, document not terminated", pi->str, pi->s);
 	    return 0;
 	    break; /* to avoid warnings */
+	case ':':
+	    if ('\0' == *pi->options->strip_ns) {
+		break;
+	    } else if ('*' == *pi->options->strip_ns && '\0' == pi->options->strip_ns[1]) {
+		start = pi->s + 1;
+	    } else if (0 == strncmp(pi->options->strip_ns, start, pi->s - start)) {
+		start = pi->s + 1;
+	    }
+	    break;
 	default:
 	    break;
 	}
@@ -810,24 +894,6 @@ read_cdata(PInfo pi) {
     }
 }
 
-inline static void
-next_non_token(PInfo pi) {
-    for (; 1; pi->s++) {
-	switch(*pi->s) {
-	case ' ':
-	case '\t':
-	case '\f':
-	case '\n':
-	case '\r':
-	case '/':
-	case '>':
-	    return;
-	default:
-	    break;
-	}
-    }
-}
-
 /* Assume the value starts immediately and goes until the quote character is
  * reached again. Do not read the character after the terminating quote.
  */
@@ -1086,10 +1152,13 @@ collapse_special(PInfo pi, char *str) {
 		    c = '?';
 		    while (';' != *s++) {
 			if ('\0' == *s) {
+			    set_error(&pi->err, "Invalid format, special character does not end with a semicolon", pi->str, pi->s);
 			    return EDOM;
 			}
 		    }
 		    s++;
+		    set_error(&pi->err, "Invalid format, invalid special character sequence", pi->str, pi->s);
+		    return 0;
 		}
 		*b++ = (char)c;
 	    }
