@@ -1,3 +1,4 @@
+
 module Ox
 
   # An Element represents a element of an XML document. It has a name,
@@ -32,30 +33,29 @@ module Ox
   #   => "Makie"
   #   doc.People.Person.age
   #   => "58"
-
   class Element < Node
     include HasAttrs
     
     # Creates a new Element with the specified name.
-    # @param [String] name name of the Element
+    # - +name+ [String] name of the Element
     def initialize(name)
       super
-      @attributes = nil
-      @nodes = nil
+      @attributes = {}
+      @nodes = []
     end
     alias name value
     
     # Returns the Element's nodes array. These are the sub-elements of this
     # Element.
-    # @return [Array] all child Nodes.
+    # *return* [Array] all child Nodes.
     def nodes
-      return [] if !instance_variable_defined?(:@nodes) or @nodes.nil?
+      @nodes = [] if !instance_variable_defined?(:@nodes) or @nodes.nil?
       @nodes
     end
 
     # Appends a Node to the Element's nodes array. Returns the element itself
     # so multiple appends can be chained together.
-    # @param [Node] node Node to append to the nodes array
+    # - +node+ [Node] Node to append to the nodes array
     def <<(node)
       raise "argument to << must be a String or Ox::Node." unless node.is_a?(String) or node.is_a?(Node)
       @nodes = [] if !instance_variable_defined?(:@nodes) or @nodes.nil?
@@ -65,10 +65,9 @@ module Ox
 
     # Returns true if this Object and other are of the same type and have the
     # equivalent value and the equivalent elements otherwise false is returned.
-    # @param [Object] other Object compare _self_ to.
-    # @return [Boolean] true if both Objects are equivalent, otherwise false.
+    # - +other+ [Object] Object compare _self_ to.
+    # *return* [Boolean] true if both Objects are equivalent, otherwise false.
     def eql?(other)
-      return false if (other.nil? or self.class != other.class)
       return false unless super(other)
       return false unless self.attributes == other.attributes
       return false unless self.nodes == other.nodes
@@ -83,6 +82,41 @@ module Ox
       nil
     end
 
+    # Clears any child nodes of an element and replaces those with a single Text
+    # (String) node. Note the existing nodes array is modified and not replaced.
+    # - +txt+ [String] to become the only element of the nodes array
+    def replace_text(txt)
+      raise "the argument to replace_text() must be a String" unless txt.is_a?(String)
+      @nodes.clear()
+      @nodes << txt
+    end
+
+    # Return true if all the key-value pairs in the cond Hash match the
+    # @attributes key-values.
+    def attr_match(cond)
+      cond.each_pair { |k,v| return false unless v == @attributes[k.to_sym] || v == @attributes[k.to_s] }
+      true
+    end
+
+    # Iterate over each child of the instance yielding according to the cond
+    # argument value. If the cond argument is nil then all child nodes are
+    # yielded to. If cond is a string then only the child Elements with a
+    # matching name will be yielded to. If the cond is a Hash then the
+    # keys-value pairs in the cond must match the child attribute values with
+    # the same keys. Any other cond type will yield to nothing.
+    def each(cond=nil)
+      if cond.nil?
+        nodes.each { |n| yield(n) }
+      else
+        cond = cond.to_s if cond.is_a?(Symbol)
+        if cond.is_a?(String)
+          nodes.each { |n| yield(n) if n.is_a?(Element) && cond == n.name }
+        elsif cond.is_a?(Hash)
+          nodes.each { |n| yield(n) if n.is_a?(Element) && n.attr_match(cond) }
+        end
+      end
+    end
+
     # Returns an array of Nodes or Strings that correspond to the locations
     # specified by the path parameter. The path parameter describes the path
     # to the return values which can be either nodes in the XML or
@@ -114,30 +148,41 @@ module Ox
     # * <code>element.locate("Family/Pete/*")</code> returns all children of the Pete Element.
     # * <code>element.locate("Family/?[1]")</code> returns the first element in the Family Element.
     # * <code>element.locate("Family/?[<3]")</code> returns the first 3 elements in the Family Element.
+    # * <code>element.locate("Family/?[@age=32]")</code> returns the elements with an age attribute equal to 32 in the Family Element.
     # * <code>element.locate("Family/?/@age")</code> returns the arg attribute for each child in the Family Element.
     # * <code>element.locate("Family/*/@type")</code> returns the type attribute value for decendents of the Family.
     # * <code>element.locate("Family/^Comment")</code> returns any comments that are a child of Family.
     #
-    # @param [String] path path to the Nodes to locate
+    # - +path+ [String] path to the Nodes to locate
     def locate(path)
       return [self] if path.nil?
       found = []
       pa = path.split('/')
-      alocate(pa, found)
+      if '*' == path[0]
+        # a bit of a hack but it allows self to be checked as well
+        e = Element.new('')
+        e << self
+        e.alocate(pa, found)
+      else
+        alocate(pa, found)
+      end
       found
     end
     
     # Handles the 'easy' API that allows navigating a simple XML by
     # referencing elements and attributes by name.
-    # @param [Symbol] id element or attribute name
-    # @return [Element|Node|String|nil] the element, attribute value, or Node identifed by the name
-    # @raise [NoMethodError] if no match is found
+    # - +id+ [Symbol] element or attribute name
+    # *return* [Element|Node|String|nil] the element, attribute value, or Node identifed by the name
+    #
+    # _raise_ [NoMethodError] if no match is found
     def method_missing(id, *args, &block)
+      has_some = false
       ids = id.to_s
       i = args[0].to_i # will be 0 if no arg or parsing fails
       nodes.each do |n|
-        if (n.is_a?(Element) || n.is_a?(Instruct)) && (n.value == id || n.value == ids)
+        if (n.is_a?(Element) || n.is_a?(Instruct)) && (n.value == id || n.value == ids || name_matchs?(n.value, ids))
           return n if 0 == i
+          has_some = true
           i -= 1
         end
       end
@@ -145,11 +190,30 @@ module Ox
         return @attributes[id] if @attributes.has_key?(id)
         return @attributes[ids] if @attributes.has_key?(ids)
       end
+      return nil if has_some
       raise NoMethodError.new("#{ids} not found", name)
     end
 
-    # @param [Array] path array of steps in a path
-    # @param [Array] found matching nodes
+    # - +id+ [String|Symbol] identifer of the attribute or method
+    # - +ignored+ inc_all [Boolean]
+    # *return* true if the element has a member that matches the provided name.
+    def respond_to?(id, inc_all=false)
+      return true if super
+      id_str = id.to_s
+      id_sym = id.to_sym
+      nodes.each do |n|
+        next if n.is_a?(String)
+        return true if n.value == id_str || n.value == id_sym || name_matchs?(n.value, id_str)
+      end
+      if instance_variable_defined?(:@attributes) && !@attributes.nil?
+        return true if @attributes.has_key?(id_str)
+        return true if @attributes.has_key?(id_sym)
+      end
+      false
+    end
+
+    # - +path+ [Array] array of steps in a path
+    # - +found+ [Array] matching nodes
     def alocate(path, found)
       step = path[0]
       if step.start_with?('@') # attribute
@@ -208,6 +272,9 @@ module Ox
             match = 0 < index ? match[0..index - 1] : []
           when '>'
             match = index <= match.size ? match[index + 1..-1] : []
+          when '@'
+            k,v = step[3..-2].split('=')
+            match = match.select { |n| n.is_a?(Element) && (v == n.attributes[k.to_sym] || v == n.attributes[k]) }
           else
             raise InvalidPath.new(path)
           end
@@ -223,5 +290,13 @@ module Ox
       end
     end
 
+    private
+
+    def name_matchs?(pat, id)
+      return false unless pat.length == id.length
+      pat.length.times { |i| return false unless '_' == id[i] || pat[i] == id[i] }
+      true
+    end
+
   end # Element
 end # Ox
