@@ -1,15 +1,17 @@
+
 module Ox
   # A SAX style parse handler. The Ox::Sax handler class should be subclasses
   # and then used with the Ox.sax_parse() method. The Sax methods will then be
   # called as the file is parsed. This is best suited for very large files or
   # IO streams.<p/>
-  # @example
+  #
+  # *Example*
   # 
   #  require 'ox'
   #
   #  class MySax < ::Ox::Sax
   #    def initialize()
-  #      @element_name = []
+  #      @element_names = []
   #    end
   #
   #    def start_element(name)
@@ -29,7 +31,7 @@ module Ox
   # arguments will be Ox::Sax::Value objects. Since both the text() and the
   # value() methods are called for the same element in the XML document the the
   # text() method is ignored if the value() method is defined or public. The
-  # same is true for attr() and attr_value(). When all attribtues have been read
+  # same is true for attr() and attr_value(). When all attributes have been read
   # the attr_done() callback will be invoked.
   #
   #    def instruct(target); end
@@ -44,14 +46,18 @@ module Ox
   #    def value(value); end
   #    def start_element(name); end
   #    def end_element(name); end
+  #    def error(message, line, column); end
+  #    def abort(name); end
   #
-  # Initializing @line in the initializer will cause that variable to be updated
-  # before each callback with the XML line number. The same is true for the
-  # @column but it will be updated with the column in the XML file that is the
-  # start of the element or node just read.
+  # Initializing _line_ attribute in the initializer will cause that variable to
+  # be updated before each callback with the XML line number. The same is true
+  # for the _column_ attribute but it will be updated with the column in the XML
+  # file that is the start of the element or node just read. @pos if defined
+  # will hold the number of bytes from the start of the document.
   class Sax
     # Create a new instance of the Sax handler class.
     def initialize()
+      #@pos = nil
       #@line = nil
       #@column = nil
     end
@@ -99,6 +105,9 @@ module Ox
     
     def error(message, line, column)
     end
+
+    def abort(name)
+    end
     
   end # Sax
 end # Ox
